/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2007-2008 by TAKAGI Nobuhisa
 * 
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 */
#include <cstdio>
#include <string>
#include "toppers/diagnostics.hpp"
#include "toppers/text_line.hpp"

namespace toppers
{
  namespace
  {
    int error_abort_threshold = 100;
    int error_count = 0;
    std::string program_name( "(unknown)" );
  }

  int get_error_count()
  {
    return error_count;
  }

  int increment_error_count()
  {
    return ++error_count;
  }

  void set_program_name( char const* name )
  {
    program_name = name;
  }

  std::string const& get_program_name()
  {
    return program_name;
  }

  int set_error_abort_threshold( int thresh )
  {
    if ( thresh < 1 )
    {
      return -1;
    }
    int previous = error_abort_threshold;
    error_abort_threshold = thresh;
    return previous;
  }

  void warning( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "warning" ), msg );
  }
  void warning( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%ld: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "warning" ), msg );
  }
  void error( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "error" ), msg );
    ++error_count;
    if ( error_abort_threshold <= error_count )
    {
      throw diagnostics_error( _( "too many errors" ) );
    }
  }
  void error( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%ld: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "error" ), msg );
    ++error_count;
    if ( error_abort_threshold <= error_count )
    {
      throw diagnostics_error( _( "too many errors" ) );
    }
  }
  void fatal( const char* msg )
  {
    fprintf( stderr, "%s: %s: %s\n", program_name.c_str(), _( "error" ), msg );
    throw diagnostics_error( _( "fatal error" ) );
  }
  void fatal( text_line const& line, const char* msg )
  {
    fprintf( stderr, "%s:%s:%d: %s: %s\n", program_name.c_str(), line.file.c_str(), line.line, _( "error" ), msg );
    throw diagnostics_error( _( "fatal error" ) );
  }

}
